/**
 * @file include/i2c_driver.h - i2c single master driver definitions
 * @brief Declarations here are not meant as public API.
 */

/*
Copyright (c) 2012 David Michael Betz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#ifndef __ADC_DRIVER_H__
#define __ADC_DRIVER_H__

#include <stdint.h>

typedef enum {
    ADC_CMD_IDLE,
    ADC_CMD_INIT,
    ADC_CMD_FLUSH
} ADC_CMD;

typedef enum {
    ADC_OK = 0,
    ADC_ERR_UNKNOWN_CMD,
    ADC_ERR_SEND_HDR,
    ADC_ERR_SEND,
    ADC_ERR_RECEIVE_HDR,
    ADC_ERR_RECEIVE
} ADC_RESULT;

typedef struct {
    uint32_t scl;
    uint32_t sda;
    uint32_t addr;
    uint32_t mask;
    uint32_t ticks_per_cycle;
} ADC_INIT;

typedef struct {
    union {
        ADC_INIT *init;
        uint32_t cmd;
    };
    uint32_t sts;
	uint32_t iPtr;
	uint32_t oPtr;
	uint32_t size;
	uint16_t *buffer;
	uint32_t lock;
} ADC_MAILBOX;

#define ADC_STACK_SIZE  16

typedef struct {
    uint32_t stack[ADC_STACK_SIZE];
    volatile ADC_MAILBOX mailbox;
} ADC_STATE;

#define ADC_READ        1
#define ADC_WRITE       0

#endif
